
/*	FLIST.H		*/

/*

	FLIST header file

	Copyright (C) JXL Systems, Inc. 1987.  All rights reserved.

*/

/*	FLIST struct to describe a file */

struct	filedesc 
{
	struct	filedesc	*next;		/* next in list */
	char	name[9];				/* name */
	char	ext[4];					/* extension */
	char	attr;					/* attributes */
	unsigned int 	date;			/* DOS format */
	unsigned int	time;			/* DOS format */
	union	
	{
		unsigned long	sizelong;	/* size as a long int */
		unsigned int	sizeint[2];	/* size as two ints */
	} size;
};

/*	DTA after int 21/0x4e or 0x4f */

struct	dta 
{
	char	filler1[21];			/* reserved for DOS */
	char	attr;					/* attribute */
	unsigned int		time;		/* file time */
	unsigned int		date;		/* file date */
	unsigned int	 	size_lo;	/* low-order word of filesize */
	unsigned int		size_hi;	/* high-order word of filesize */
	char	nameext[13];			/* name and extension ASCIIZ */
};

/*	FLIST struct describing the presentation of a file-name */

struct	fmenuent 
{
	char	blank0[2];
	char	name[8];
	char	blank1;
	char	ext[3];
	char	blank2[2];
	char	null;
};

/*	Struct describing the displayable info for a file */

struct fileinfo 
{
	char	date[8];
	char	blank1;
	char	time[6];
	char	blank2;
	char	size[8];
	char	blank3;
	char	blank4;
	char	arc;
	char	blank5;
	char	blank6;
	char	blank7[2];
	char	rdonly;
	char	ff;
	char	c0;
};

/*	Struct describing the header for displayable info for a file */

struct fileihdr 
{
	char	date[4];
	char	blank1[5];
	char	time[4];
	char	blank2[3];
	char	size[4];
	char	blank3[5];
	char	arc[3];
	char	blank6;
	char	rdonly[6];
	char	ff;
	char	c0;
};

/*  Struct describing the printable info for a file */

struct fileprt 
{
	char	name[8];
	char	blank1;
	char	ext[3];
	char	blank2[2];
	char	date[8];
	char	blank3;   
	char	time[6];
	char	blank4;
	char	size[8];
	char	blank5[2];
	char	arc;
	char	blank6[4];
	char	rdonly;
	char	nl;
	char	null;
};

/*  Struct describing the header for printable file info */

struct filephdr 
{
	char	blank1[4];
	char	name[4];
	char	blank2[8];
	char	date[4];
	char	blank3[4];
	char	time[4];
	char	blank4[4];
	char	size[4];
	char	blank5[3];
	char	arc[3];
	char	blank6;
	char	rdonly[6];
	char	nl1;
	char	nl2;
	char	null;
};

/*	Struct describing edit or browse help */

struct	ebhelp	
{
	char	*window_name;
	char	*help_text;
};

/*	breakdown of dta_pointer */

union dtaptr_breakdown 
{
	struct dta *pointer;
	int components[2];
};

/*	file date & time */

struct filedate 
{
#ifdef LATTC3
	unsigned year : 7;
	unsigned month: 4;
	unsigned day: 5;
#else
	unsigned day : 5;
	unsigned month : 4;
	unsigned year : 7;
#endif
};

struct filetime 
{
#ifdef	LATTC3
	unsigned hour : 5;
	unsigned minute : 6;
	unsigned second : 5;
#else
	unsigned second : 5;
	unsigned minute : 6;
	unsigned hour : 5;
#endif
};

union filedttm 
{
	int integer;
	struct filedate date;
	struct filetime time;
};

/*	Some function-protos that will be referenced in external-data */

int compname(struct filedesc **,struct filedesc **);
int compext(struct filedesc **,struct filedesc **);
int compdate(struct filedesc **,struct filedesc **);
int compsize(struct filedesc **,struct filedesc **);


/*	External Data declarations	*/

#ifndef	MAIN

extern Menu					*mainmenu;	/* main menu descriptor */
extern Menu					*dirmenu;	/* directory-action menu descriptor */
extern Menu					*dirlist;	/* directory list */
extern Menu					*sortmenu;	/* directory-sorting menu */
extern Menu					*helpmenu;	/* main help menu descriptor */
extern Proctbl			 	*ipmain;	/* main menu select list */
extern Proctbl 				*ipsort;	/* sort menu select list */
extern Proctbl				*iphelp;	/* help menu select list */
extern Proctbl				*ipdir;		/* directory-act menu select list */
extern Freeform				*kedit;		/* edit control block */
extern Freeform				*kbrowse;	/* browse control block */
extern Structin				*kcopy;		/* copyfile control block */
extern Structin				*krename;	/* rename-file control block */
extern Structin				*kcreate;	/* create-file control block */
extern Structin				*kfind;		/* edit/browse "find" */
extern Structin				*krfind;	/* edit "find for replace" */
extern Structin				*kreplace;	/* edit "replace" */
extern Structin				*kdirname;	/* directory-name */
extern Structin				*keditcon;	/* confirm edit-abandon */
extern Structin				*kcopycon;	/* confirm copy-overwrite */
extern Structin				*kdelcon; 	/* confirm delete */
extern char					pathn[50];  /* current directory pathname */
extern char					pathpart[]; /* path part of name */
extern char					fullfn[144];
extern char					fullpath[130];
extern char					drive[3];
extern char					dir[130];
extern char					fnm[9];
extern char					ext[5];
extern char					fullfn2[144];
extern char					fullpath2[130];
extern char					drive2[3];
extern char					dir2[130];
extern char					fnm2[9];
extern char					ext2[5];
extern char		edfile_name[9];			/* file to be edited */
extern char		edfile_ext[4];			/* file-extension to be edited */
extern int					iocancel;	/* cancel on I/O error */
extern int					criterr;	/* critical error occurred */
extern int					cn_ok;		/* create-new go/nogo */
extern struct	filedesc	*filelist;	/* list of files in current direc */
extern struct	filedesc	**filevec;	/* vectors to above list */
extern int					numfiles;	/* number of files */
extern struct	filedesc	*selfile;	/* selected file */
extern char					file2[];   	/* to-file in rename or copy */
extern FILE					*fd1;		/* first FILE pointer */
extern FILE					*fd2;		/* second FILE pointer */
extern char					*fb1;		/* first file buffer */
extern char					*fb2;		/* second file buffer */
extern union	REGS		inregs;
extern union	REGS		outregs;
extern struct	SREGS		segregs;
extern struct	fmenuent	fmenuent;	/* file menu entry */
extern struct	fileinfo	fileinfo;	/* file info */
extern struct	fileihdr 	fileihdr;	/* file info header */
extern struct	fileprt		fileprt;	/* printable file info */
extern struct	filephdr	filephdr;	/* printable file info header */
extern struct	dta			*dtaptr;	/* pointer to DOS DTA struct */
extern int					sortflag;	/* indicates directory was sorted */
extern int					conflag;	/* action-confirmed flag */
extern int					kcode;		/* endcode for keyin operations */
extern int					kpfunret;	/* retcode for keyin operations */
extern struct	ebhelp		edhelp[];	/* edit keystroke help */
extern struct	ebhelp		brhelp[];	/* browse keystroke help */
extern union dtaptr_breakdown  ptr_to_dta;
extern char 				wkarea[];
extern union filedttm		filedt;		/* file date & time */
extern unsigned int 		fileyear;
extern unsigned int		filemonth;
extern unsigned int		fileday;
extern unsigned int		filehour;
extern unsigned int		fileminute;
extern unsigned int		file12h;
extern char				ampm;
extern int (*compfuncs[])();
#endif

#ifdef	MAIN

Menu				*mainmenu;	/* main menu descriptor */
Menu				*dirmenu;	/* directory-action menu descriptor */
Menu				*dirlist;	/* directory list */
Menu				*sortmenu;	/* directory-sorting menu */
Menu				*helpmenu;	/* main help menu descriptor */
Proctbl        		*ipmain;	/* main menu select list */
Proctbl        		*ipsort;	/* sort menu select list */
Proctbl				*iphelp;	/* help menu select list */
Proctbl				*ipdir;		/* directory-act menu select list */
Freeform			*kedit;		/* edit control block */
Freeform			*kbrowse;	/* browse control block */
Structin			*kcopy;		/* copyfile control block */
Structin			*krename;	/* rename-file control block */
Structin			*kcreate;	/* create-file control block */
Structin			*kfind;		/* edit/browse "find" */
Structin			*krfind;	/* edit "find for replace" */
Structin			*kreplace;	/* edit "replace" */
Structin			*kdirname;	/* directory-name */
Structin			*keditcon;	/* confirm edit-abandon */
Structin			*kcopycon;	/* confirm copy-overwrite */
Structin			*kdelcon; 	/* confirm delete */
char				pathn[50];  /* current directory pathname */
char				pathpart[50]; /* path part of chosen dir. name */
char					fullfn[144];
char					fullpath[130];
char					drive[3];
char					dir[130];
char					fnm[9];
char					ext[5];
char					fullfn2[144];
char					fullpath2[130];
char					drive2[3];
char					dir2[130];
char					fnm2[9];
char					ext2[5];
char	edfile_name[9];			/* file to be edited */
char	edfile_ext[4];			/* extension of file to be edited */
int					iocancel;	/* cancel on I/O error */
/* The following item is initialized just to force it into the DATA seg. */
int					criterr = 1; /* critical error occurred */
int					cn_ok;		/* create-new go/nogo */
struct	filedesc	*filelist;	/* list of files in current direc */
struct	filedesc	**filevec;	/* vectors to above list */
int					numfiles;	/* number of files */
struct	filedesc	*selfile;	/* selected file */
char				file2[144];	/* to-file in rename or copy */
FILE				*fd1;		/* first FILE pointer */
FILE				*fd2;		/* second FILE pointer */
char				*fb1;		/* first file buffer */
char				*fb2;		/* second file buffer */
union	REGS		inregs;
union	REGS		outregs;
struct	SREGS		segregs;
struct	fmenuent	fmenuent;	/* file menu entry */
struct	fileinfo	fileinfo;	/* file info */
struct	fileihdr 	fileihdr;	/* file info header */
struct	fileprt		fileprt;	/* printable file info */
struct	filephdr	filephdr;	/* printable file info header */
struct	dta			*dtaptr;	/* DOS DTA struct */
int					sortflag;	/* indicates directory was sorted */
int					conflag;	/* action-confirmed flag */
int					kcode;		/* endcode for keyin operations */
int					kpfunret;	/* retcode for keyin operations */
union dtaptr_breakdown  ptr_to_dta;
char				wkarea[50];
union 	filedttm	filedt;		/* file date & time */
unsigned int 		fileyear;
unsigned int		filemonth;
unsigned int		fileday;
unsigned int		filehour;
unsigned int		fileminute;
unsigned int		file12h;
char				ampm;
int (*compfuncs[])() = { compname, compext, compdate, compsize };

#endif

/*	Function declarations */

int	mmentry(void);
int mmres(void);
int smentry(void);
int smexit(void);
int dientry(void);
int diexit(void);
int dlentry(void);
int dlexit(void);
int dlsusp(void);
int dlres(void);
int rpentry(void);
int rpexit(void);
int cpentry(void);
int cpexit(void);
int cpres(void);
int rnentry(void);
int rnexit(void);
int dninit(void);
int dnend(void);
int cnentry(void);
int cnexit(void);
int dl(void);
int mm(void);
int di(void);
int sm(void);
int ed(void);
int br(void);
int cf(void);
int rf(void);
int dn(void);
int cn(void);

void getdtptr(void);
void clrstat(void);
void menuinst(void);
void getfldat(void);
void fmtdt(void);
void fmttm(void);
void fmtsize(void);
void outofmem(void);
void f1hlp(void);
void toobig(void);
void edbrhelp(struct ebhelp *);
int  confirm(char *);
int  x_int23h(void);
int  x_int24h(void);
int cname(struct filedesc **,struct filedesc **);
int cext(struct filedesc **,struct filedesc **);
int cdate(struct filedesc **,struct filedesc **);
int csize(struct filedesc **,struct filedesc **);

#ifdef	MAIN

/*	Keystroke-definition tables */

Proctbl ffedtbl[] =    /* table for freeform edit */

{

                    { Ks_FfE_CursorToTopOfPage(N_MINUS) },
                    { Ks_FfE_CursorUp(UPARROW) },
                    { Ks_FfE_CursorDown(DOWNARROW) },
                    { Ks_FfE_ScrollUpPage(PGUP) },
                    { Ks_FfE_CursorLeft(LEFTARROW) },
                    { Ks_FfE_CursorRight(RIGHTARROW) },
                    { Ks_FfE_CursorToBottomOfPage(N_PLUS) },
                    { Ks_FfE_ScrollDownPage(PGDN) },
                    { Ks_FfE_ScrollHalfPageLeft(N_4) },
                    { Ks_FfE_ScrollHalfPageRight(N_6) },
                    { Ks_FfE_ToggleInsertMode(INSERT) },
                    { Ks_FfE_DeleteCharUnderCursor(DELETE) },
                    { Ks_FfE_CursorToTopOfFile(C_HOME) },
                    { Ks_FfE_ScrollUpOneLine(C_PGUP) },
                    { Ks_FfE_CursorToEndOfFile(C_END) },
                    { Ks_FfE_ScrollDownOneLine(C_PGDN) },
                    { Ks_FfE_GotoNextTabStop(TAB) },
                    { Ks_FfE_DeleteCharToLeftOfCursor(BACKSPACE) },
                    { Ks_FfE_NewLine(ENTER) },
                    { Ks_FfE_InterruptEdit(ESC) },
                    { Ks_FfE_InterruptEdit(A_Q) },
                    { Ks_FfE_InterruptEdit(A_K) },
                    { Ks_FfE_InterruptEdit(U_F1) },
                    { Ks_FfE_CursorToStartOfLine(HOME) },
                    { Ks_FfE_CursorToEndOfLine(END) },
                    { Ks_FfE_DeleteLine(C_BACKSPACE) },
                    { Ks_FfE_DeleteToEndOfLine(N_PERIOD) },
                    { Ks_FfE_MarkStartOfBlock(A_S) },
                    { Ks_FfE_MarkEndOfBlock(A_E) },
                    { Ks_FfE_CopyMarkedBlock(A_C) },
                    { Ks_FfE_MoveMarkedBlock(A_M) },
                    { Ks_FfE_DeleteMarkedBlock(A_D) },
                    { Ks_FfE_UnmarkBlock(A_U) },
                    { Ks_FfE_SetTabStops(A_T) },
                    { Ks_FfE_Find(A_F) },
                    { Ks_FfE_FindAndReplace(A_R) },
                    { Ks_FfE_RepeatLastFindOrReplace(A_P) },
                    { Ks_FfE_IncludeFile(A_I) },
                    { Ks_FfE_WriteMarkedBlockToFile(A_W) },
					{ Ks_FfE_ToggleAutoIndentMode(A_A) },
					{ Ks_FfE_UnDelete(N_0) },
                    { Ks_EndOfList }
};

Proctbl ffbrtbl[] =    /* table for freeform browse */

{
                    { Ks_FfB_CursorToTopOfPage(N_MINUS) },
                    { Ks_FfB_CursorUp(UPARROW) },
                    { Ks_FfB_CursorDown(DOWNARROW) },
                    { Ks_FfB_CursorLeft(LEFTARROW) },
                    { Ks_FfB_CursorRight(RIGHTARROW) },
                    { Ks_FfB_CursorToBottomOfPage(N_PLUS) },
                    { Ks_FfB_ScrollUpPage(PGUP) },
                    { Ks_FfB_ScrollDownPage(PGDN) },
                    { Ks_FfB_ScrollHalfPageLeft(N_4) },
                    { Ks_FfB_ScrollHalfPageRight(N_6) },
                    { Ks_FfB_ScrollToTopOfFile(C_HOME) },
                    { Ks_FfB_ScrollUpOneLine(C_PGUP) },
                    { Ks_FfB_ScrollToEndOfFile(C_END) },
                    { Ks_FfB_ScrollDownOneLine(C_PGDN) },
                    { Ks_FfB_InterruptBrowse(ESC) },
                    { Ks_FfB_InterruptBrowse(A_Q) },
                    { Ks_FfB_InterruptBrowse(U_F1) },
                    { Ks_FfB_ScrollToStartOfLine(HOME) },
                    { Ks_FfB_ScrollToEndOfLine(END) },
                    { Ks_FfB_SetTabStops(A_T) },
                    { Ks_FfB_Find(A_F) },
                    { Ks_FfB_RepeatLastFind(A_P) },
                    { Ks_EndOfList }
};

Proctbl stedtbl[] =   /* table for structured edit */

{

                    { Ks_StE_CursorLeft(LEFTARROW) },
                    { Ks_StE_CursorRight(RIGHTARROW) },
                    { Ks_StE_ToggleInsertMode(INSERT) },
                    { Ks_StE_DeleteCharUnderCursor(DELETE) },
                    { Ks_StE_DeleteCharToLeftOfCursor(BACKSPACE) },
                    { Ks_StE_CursorToStartOfLine(HOME) },
                    { Ks_StE_CursorToEndOfLine(END) },
                    { Ks_StE_DeleteLine(C_BACKSPACE) },
                    { Ks_StE_DeleteToEndOfLine(N_PERIOD) },
                    { Ks_StE_InterruptEdit(ENTER) },
                    { Ks_StE_InterruptEdit(ESC) },
                    { Ks_EndOfList }
};

struct ebhelp edhelp[] = /*	Edit help-text & subwindows */

{
	
	{ "a00jaaa" , "Alt-K   Keep changes & exit\xff\xc0" },
	{ "a00jaba" , "Alt-Q or Esc Quit, forget it\xff\xc0" },
	{ "a00jaca" , "Shf Ins undo line/blk delete\xff\xc0" },
	{ "a00jada" , "^Home   start of file\xff\xc0" },
	{ "a00jaea" , "^End    end of file\xff\xc0" },
	{ "a00jafa" , "Alt-F   Find\xff\xc0" },
	{ "a00jaga" , "Alt-R   find and Replace\xff\xc0" },
	{ "a00jaha" , "^Pgup   scroll up one line\xff\xc0" },
	{ "a00jaia" , "^Pgdn   scroll down one line\xff\xc0" },
	{ "a00jaja" , "Alt-P   rPt last find/replace\xff\xc0" },
	{ "a00jaka" , "Alt-I   Include a file\xff\xc0" },
	{ "a00jala" , "Alt-A   toggle Autoindent\xff\xc0" },
	{ "a00jama" , "Gray -  csr to top of window\xff\xc0" },
	{ "a00jana" , "Gray +  csr to end of window\xff\xc0" },
	{ "a00jaoa" , "Home    csr to left of line\xff\xc0" },
	{ "a00japa" , "End     csr to right of line\xff\xc0" },
	{ "a00jaqa" , "^Bksp   delete line\xff\xc0" },
	{ "a00jara" , "Shf Del delete to end of line\xff\xc0" },
	{ "a00jasa" , "Alt-S   mark Start of block\xff\xc0" },
	{ "a00jata" , "Alt-E   mark End of block\xff\xc0" },
	{ "a00jaua" , "Alt-C   Copy marked block \x1a csr\xff\xc0" },
	{ "a00java" , "Alt-M   Move marked block \x1a csr\xff\xc0" },
	{ "a00jawa" , "Alt-D   Delete marked block\xff\xc0" },
	{ "a00jaxa" , "Alt-U   Unmark block\xff\xc0" },
	{ "a00ja5a" , "Shf \x1b   scroll left half page\xff\xc0" },
	{ "a00ja6a" , "Shf \x1a   scroll right half page\xff\xc0" },
	{ "a00ja7a" , "Alt-T   set Tab stops\xff\xc0" },
	{ "a00ja8a" , "Alt-W   Write mkd blk to file\xff\xc0" },
	{ NULL, NULL }
	
};

struct	ebhelp	brhelp[] = /*	Browse help/text & subwindows */

{
		
	{ "a00jaaa" , "Alt-Q or Esc  Quit\xff\xc0" },
	{ "a00jaca" , "^Home   start of file\xff\xc0" },
	{ "a00jada" , "^End    end of file\xff\xc0" },
	{ "a00jaea" , "Alt-F   Find\xff\xc0" },
	{ "a00jafa" , "^Pgup   scroll up one line\xff\xc0" },
	{ "a00jaga" , "^Pgdn   scroll down one line\xff\xc0" },
	{ "a00jaha" , "Gray -  csr to top of window\xff\xc0" },
	{ "a00jaia" , "Gray +  csr to end of window\xff\xc0" },
	{ "a00jaja" , "Home    csr to left of line\xff\xc0" },
	{ "a00jaka" , "End     csr to right of line\xff\xc0" },
	{ "a00jala" , "Shf \x1b   scroll left half page\xff\xc0" },
	{ "a00jama" , "Shf \x1a   scroll right half page\xff\xc0" },
	{ "a00jana" , "Alt-T   set Tab stops\xff\xc0" },
	{ "a00jaoa" , "Alt-P   rePeat last find\xff\xc0" },
	{ NULL, NULL }
	
};
	
#endif

/*	Symbolic names for Screen Subdivisions */

#define	CopyrightWindow				"aa00a"
#define	MainMenuWindow				"ab00aaa"
#define	StatusWindow_1				"ab00aba"
#define	StatusWindow_2				"ab00aca"
#define	StatusWindow_3				"ab00ada"
#define	FunctionDisplayWindow		"ac00a"
#define	DirectoryDisplayWindow		"ad00a"
#define	BrowseWindow				"ad00b"
#define	EditWindow					"ad00c"
#define	InstructionsWindow			"ae00a"
#define	InstructionsWindow_1		"ae00aaa"
#define	InstructionsWindow_2		"ae00aba"
#define	InstructionsWindow_3		"ae00aca"
#define	SortMenuPopup				"a00aa"
#define	SortMenuWindow				"a00aa"
#define	ActionMenuPopup				"a00ba"
#define	ActionMenu_1				"a00baaa"
#define	ActionMenu_2				"a00baba"
#define	ActionMenuWindow			"a00baca"
#define	CopyToFilenamePopup			"a00ca"
#define	CopyToFilename_1			"a00caaa"
#define	CopyToFilenameWindow		"a00caba"
#define	RenameToFilenamePopup		"a00da"
#define	RenameToFilename_1			"a00daaa"
#define	RenameToFilenameWindow		"a00daba"
#define	DirectoryNamePopup			"a00ea"
#define	DirectoryName_1				"a00eaaa"
#define	DirectoryName_2				"a00eaca"
#define	DirectoryName_3				"a00eada"
#define	DirectoryNameWindow			"a00eaba"
#define	FileInfoPopup				"a00fa"
#define	FileInfo_1					"a00faaa"
#define	FileInfo_2					"a00faba"
#define	FileInfo_3					"a00faca"
#define	FileInfo_4					"a00fada"
#define	FileInfo_5					"a00faea"
#define	EditHelpPopup				"a00ja"
#define	EditHelp_1					"a00ja1a"
#define	EditHelp_2					"a00ja4a"
#define	CreateNewFilePopup			"a00la"
#define	CreateNewFile_1				"a00laaa"
#define	CreateNewFileWindow			"a00laba"

/*	Symbolics for Menu Selections */

#define	MainChooseDirectory			1
#define	MainQuit					2

#define	SortByFilename				1
#define	SortByExtension				(SortByFilename + 1)
#define	SortByDate					(SortByFilename + 2)
#define	SortBySize					(SortByFilename + 3)

#define	FileSeeInfo					1
#define	FileDelete					2
#define	FileCopy					3
#define	FilePrint					4
#define	FileRename					5
#define	FileBrowse					6 
#define	FileEdit					7
